==> catberry-oauth2-client-2.0.8.tgz: package/lib/endpoints/PasswordCredentialsFlowEndpoint.js <==
/*
 * catberry-oauth
 *
 * Copyright (c) 2014 Denis Rechkunov and project contributors.
 *
 * catberry-oauth's license follows:
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * This license applies to all parts of catberry-oauth that are not externally
 * maintained libraries.
 */

'use strict';

module.exports = PasswordCredentialsFlowEndpoint;

var util = require('util'),
	httpHelper = require('../helpers/httpHelper'),
	Query = require('catberry-uri').Query,
	PasswordCredentialsSender = require('../grants/PasswordCredentialsSender'),
	GrantFlowMiddlewareBase = require('../middlewares/GrantFlowMiddlewareBase');

var CONTENT_TYPE = 'application/x-www-form-urlencoded',
	TRACE_OBTAIN_TOKEN = 'Obtaining access token for password credentials...',
	MAX_UPSTREAM_LENGTH = 512, // 0.5KB
	FIELD_USERNAME = 'username',
	FIELD_PASSWORD = 'password',
	FIELD_SCOPE = 'scope',
	ERROR_USERNAME = '"username" is required parameter',
	ERROR_PASSWORD = '"password" is required parameter';

util.inherits(PasswordCredentialsFlowEndpoint, GrantFlowMiddlewareBase);

/**
 * Creates new instance of password credentials flow endpoint.
 * @param {ServiceLocator} $serviceLocator Service locator to resolve
 * dependencies.
 *
 * @param {Object} authConfig Authorization configuration.
 * @param {String} authConfig.clientId Id of OAuth 2.0 client.
 * @param {String} authConfig.clientSecret Secret of OAuth 2.0 client.
 * @param {String} authConfig.authServerUrl URL to OAuth 2.0
 * authentication server.
 * @param {Number?} authConfig.timeout Grant send timeout (30000 ms by default).
 * @param {Boolean?} authConfig.unsafeHTTPS Determines if sender will send grant
 * via unsafe HTTPS connection with invalid certificate (false by default).
 * @param {String?} authConfig.tokenEndpointPath URL path to
 * OAuth 2.0 token endpoint (/token by default).
 *
 * @param {Object} endpointConfig Endpoint configuration.
 * @param {String?} endpointConfig.scope Access scope for endpoint.
 * @param {Object} endpointConfig.cookie Token cookie configuration.
 * @param {String} endpointConfig.cookie.accessTokenName Name of cookie
 * with access token.
 * @param {String?} endpointConfig.cookie.accessTokenExpiresIn Expiration time
 * in seconds for access token cookie if it is not specified by authorization
 * server (3 600 secs by default, 1 hour).
 * @param {String} endpointConfig.cookie.refreshTokenName Name of cookie
 * with refresh token.
 * @param {String?} endpointConfig.cookie.refreshTokenExpiresIn Expiration time
 * in seconds for refresh token cookie
 * (3 110 400 000 secs by default, 100 years).
 * @param {String?} endpointConfig.cookie.path Path attribute for cookie
 * ('/' by default).
 * @param {String?} endpointConfig.cookie.domain Domain attribute for cookie.
 * @constructor
 * @extends GrantFlowMiddlewareBase
 */
function PasswordCredentialsFlowEndpoint($serviceLocator, authConfig,
	endpointConfig) {
	GrantFlowMiddlewareBase.call(this, $serviceLocator, endpointConfig);
	this._sender = new PasswordCredentialsSender(
		$serviceLocator, authConfig
	);
}

/**
 * Current password credentials sender.
 * @type {PasswordCredentialsSender}
 * @private
 */
PasswordCredentialsFlowEndpoint.prototype._sender = null;

/**
 * Handles middleware endpoint.
 * @param {http.IncomingMessage} request HTTP request.
 * @param {http.ServerResponse} response HTTP response.
 */
PasswordCredentialsFlowEndpoint.prototype.handler =
	function (request, response) {
		this._logger.trace(TRACE_OBTAIN_TOKEN);
		var self = this;
		this._obtainAuthorization(request)
			.then(function (issuedAuth) {
				return self._handleIssuedAuthorization(
					request, response, issuedAuth
				);
			})
			.then(function (responseObject) {
				httpHelper.writeToResponse(200, responseObject, response);
			})
			.then(null, function (reason) {
				self._logger.error(reason);
				httpHelper.writeError(reason, response);
			});
	};

/**
 * Obtains authorization from authorization server.
 * @param {http.IncomingMessage} request Incoming HTTP request.
 * @returns {Promise<Object>} Promise for issued authorization.
 * @private
 */
PasswordCredentialsFlowEndpoint.prototype._obtainAuthorization =
	function (request) {
		var self = this;
		var methodError = httpHelper.checkMethod(request, 'post');
		if (methodError) {
			return Promise.reject(methodError);
		}
		var contentTypeError = httpHelper.checkContentType(
			request, CONTENT_TYPE
		);
		if (contentTypeError) {
			return Promise.reject(contentTypeError);
		}

		return new Promise(function (fulfill, reject) {
			var upstream = '';
			request.setEncoding('utf8');
			request
				.on('data', function (chunk) {
					var lengthError = httpHelper.checkLength(
						upstream, chunk, MAX_UPSTREAM_LENGTH
					);
					if (lengthError) {
						reject(lengthError);
						return;
					}
					upstream += chunk;
				})
				.on('error', function (error) {
					reject(error);
				})
				.on('end', function () {
					var parsed = new Query(upstream.replace('+', '%20')).values,
						username = parsed[FIELD_USERNAME],
						password = parsed[FIELD_PASSWORD],
						scope = parsed[FIELD_SCOPE] || self._scope;

					if (typeof(username) !== 'string') {
						var usernameError = new Error(ERROR_USERNAME);
						usernameError.code = 400;
						return reject(usernameError);
					}
					if (typeof(password) !== 'string') {
						var passwordError = new Error(ERROR_PASSWORD);
						passwordError.code = 400;
						return reject(passwordError);
					}

					self._sender.send(
						username, password, scope
					)
						.then(fulfill, reject);
				});
		});
	};

==> catberry-oauth2-client-2.0.10.tgz: package/lib/endpoints/PasswordCredentialsFlowEndpoint.js <==
/*
 * catberry-oauth
 *
 * Copyright (c) 2014 Denis Rechkunov and project contributors.
 *
 * catberry-oauth's license follows:
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * This license applies to all parts of catberry-oauth that are not externally
 * maintained libraries.
 */

'use strict';

module.exports = PasswordCredentialsFlowEndpoint;

var util = require('util'),
	httpHelper = require('../helpers/httpHelper'),
	Query = require('catberry-uri').Query,
	PasswordCredentialsSender = require('../grants/PasswordCredentialsSender'),
	GrantFlowMiddlewareBase = require('../middlewares/GrantFlowMiddlewareBase');

var CONTENT_TYPE = 'application/x-www-form-urlencoded',
	TRACE_OBTAIN_TOKEN = 'Obtaining access token for password credentials...',
	MAX_UPSTREAM_LENGTH = 512, // 0.5KB
	FIELD_USERNAME = 'username',
	FIELD_PASSWORD = 'password',
	FIELD_SCOPE = 'scope',
	ERROR_USERNAME = '"username" is required parameter',
	ERROR_PASSWORD = '"password" is required parameter';

util.inherits(PasswordCredentialsFlowEndpoint, GrantFlowMiddlewareBase);

/**
 * Creates new instance of password credentials flow endpoint.
 * @param {ServiceLocator} $serviceLocator Service locator to resolve
 * dependencies.
 *
 * @param {Object} authConfig Authorization configuration.
 * @param {string} authConfig.clientId Id of OAuth 2.0 client.
 * @param {string} authConfig.clientSecret Secret of OAuth 2.0 client.
 * @param {string} authConfig.authServerUrl URL to OAuth 2.0
 * authentication server.
 * @param {number?} authConfig.timeout Grant send timeout (30000 ms by default).
 * @param {boolean?} authConfig.unsafeHTTPS Determines if sender will send grant
 * via unsafe HTTPS connection with invalid certificate (false by default).
 * @param {string?} authConfig.tokenEndpointPath URL path to
 * OAuth 2.0 token endpoint (/token by default).
 *
 * @param {Object} endpointConfig Endpoint configuration.
 * @param {string?} endpointConfig.scope Access scope for endpoint.
 * @param {Object} endpointConfig.cookie Token cookie configuration.
 * @param {string} endpointConfig.cookie.accessTokenName Name of cookie
 * with access token.
 * @param {string?} endpointConfig.cookie.accessTokenExpiresIn Expiration time
 * in seconds for access token cookie if it is not specified by authorization
 * server (3 600 secs by default, 1 hour).
 * @param {string} endpointConfig.cookie.refreshTokenName Name of cookie
 * with refresh token.
 * @param {string?} endpointConfig.cookie.refreshTokenExpiresIn Expiration time
 * in seconds for refresh token cookie
 * (3 110 400 000 secs by default, 100 years).
 * @param {string?} endpointConfig.cookie.path Path attribute for cookie
 * ('/' by default).
 * @param {string?} endpointConfig.cookie.domain Domain attribute for cookie.
 * @constructor
 * @extends GrantFlowMiddlewareBase
 */
function PasswordCredentialsFlowEndpoint(
		$serviceLocator, authConfig, endpointConfig
	) {
	GrantFlowMiddlewareBase.call(this, $serviceLocator, endpointConfig);
	this._sender = new PasswordCredentialsSender(
		$serviceLocator, authConfig
	);
}

/**
 * Current password credentials sender.
 * @type {PasswordCredentialsSender}
 * @private
 */
PasswordCredentialsFlowEndpoint.prototype._sender = null;

/**
 * Handles middleware endpoint.
 * @param {http.IncomingMessage} request HTTP request.
 * @param {http.ServerResponse} response HTTP response.
 */
PasswordCredentialsFlowEndpoint.prototype.handler =
	function (request, response) {
		this._logger.trace(TRACE_OBTAIN_TOKEN);
		var self = this;
		this._obtainAuthorization(request)
			.then(function (issuedAuth) {
				return self._handleIssuedAuthorization(
					request, response, issuedAuth
				);
			})
			.then(function (responseObject) {
				httpHelper.writeToResponse(200, responseObject, response);
			})
			.then(null, function (reason) {
				self._logger.error(reason);
				httpHelper.writeError(reason, response);
			});
	};

/**
 * Obtains authorization from authorization server.
 * @param {http.IncomingMessage} request Incoming HTTP request.
 * @returns {Promise<Object>} Promise for issued authorization.
 * @private
 */
PasswordCredentialsFlowEndpoint.prototype._obtainAuthorization =
	function (request) {
		var self = this;
		var methodError = httpHelper.checkMethod(request, 'post');
		if (methodError) {
			return Promise.reject(methodError);
		}
		var contentTypeError = httpHelper.checkContentType(
			request, CONTENT_TYPE
		);
		if (contentTypeError) {
			return Promise.reject(contentTypeError);
		}

		return new Promise(function (fulfill, reject) {
			var upstream = '';
			request.setEncoding('utf8');
			request
				.on('data', function (chunk) {
					var lengthError = httpHelper.checkLength(
						upstream, chunk, MAX_UPSTREAM_LENGTH
					);
					if (lengthError) {
						reject(lengthError);
						return;
					}
					upstream += chunk;
				})
				.on('error', function (error) {
					reject(error);
				})
				.on('end', function () {
					var parsed = new Query(upstream.replace('+', '%20')).values,
						username = parsed[FIELD_USERNAME],
						password = parsed[FIELD_PASSWORD],
						scope = parsed[FIELD_SCOPE] || self._scope;

					if (typeof (username) !== 'string') {
						var usernameError = new Error(ERROR_USERNAME);
						usernameError.code = 400;
						return reject(usernameError);
					}
					if (typeof (password) !== 'string') {
						var passwordError = new Error(ERROR_PASSWORD);
						passwordError.code = 400;
						return reject(passwordError);
					}

					self._sender.send(
						username, password, scope
					)
						.then(fulfill, reject);
				});
		});
	};
